# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class BactecMGITSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class BactecMGITSegmentP(Segment):
    seq_number = HprimString(2)

    orders = HprimForeign("O", "patient")


class BactecMGITSegmentO(Segment):
    accession_number = HprimString((3, 1))
    isolate_number = HprimString((3, 2))
    test_id = HprimString((5, 4))

    results = HprimForeign("R", "order")


class BactecMGITSegmentR(Segment):
    result_type_code = HprimString((3, 4))
    test_number = HprimString((3, 5))
    antibiotic = HprimString((3, 6))
    antibiotic_conc = HprimString((3, 7))
    antibiotic_conc_unit = HprimString((3, 8))

    test_status = HprimString((4, 1))
    growth_units = HprimString((4, 2))
    ast_susceptibility = HprimString((4, 3))

    status = HprimString(9)
    test_start_date = HprimDate(12)
    result_status_date = HprimString(13)

    instrument_type = HprimString((14, 1))
    protocol_length = HprimString((14, 3))
    instrument_number = HprimString((14, 4))
    instrument_location = HprimString((14, 5))


class BactecMGITSegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {"code": "bactec_mgit", "label": "Bactec MGIT 960 BD"}

    SEGMENT_CLASS_MAP = {
        "H": BactecMGITSegmentH,
        "P": BactecMGITSegmentP,
        "O": BactecMGITSegmentO,
        "R": BactecMGITSegmentR,
        "L": BactecMGITSegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    # MGIT= Mycobacterium Growth Tuberculosis
    # GND = Growth And Detection
    # AST = Antimycobacterial Susceptibility Testing

    OPTIONS = [
        AstmMessageV2.Option(
            "bactec_hour_min_sep",
            label="Le séparateur entre les heures et les minutes",
            default="H",
        )
    ]

    def __init__(self, **kwargs):
        self.bactec_hour_min_sep = kwargs.pop("bactec_hour_min_sep", "H")
        super(Driver, self).__init__(**kwargs)

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    type="patient", sample_id=seg_o.accession_number
                )
                liste_atb = []
                for seg_r in seg_o.results:
                    if seg_r.result_status_date:
                        date = datetime.datetime.strptime(
                            seg_r.result_status_date, "%Y%m%d%H%M%S"
                        )
                    else:
                        date = None
                    growth_delay = "0"
                    if seg_r.result_type_code.startswith("GND"):
                        sample.rack = seg_r.instrument_location
                        sample.results.add(
                            code=seg_r.result_type_code,
                            value=seg_r.test_status,
                            date=date,
                            instrument="{}-{}".format(
                                seg_r.instrument_type, seg_r.instrument_number
                            ),
                        )
                        if seg_r.test_number:
                            sample.results.add(
                                code="BOTTLE_ID",
                                value=seg_r.test_number,
                                date=date,
                                instrument="{}-{}".format(
                                    seg_r.instrument_type, seg_r.instrument_number
                                ),
                            )
                        if seg_r.instrument_location:
                            sample.results.add(
                                code="BOTTLE_RACK",
                                value=seg_r.instrument_location,
                                date=date,
                                instrument="{}-{}".format(
                                    seg_r.instrument_type, seg_r.instrument_number
                                ),
                            )
                        # Date et heure d'introduction du flacon
                        if seg_r.test_start_date:
                            sample.results.add(
                                code="INTRODUCE_DATETIME",
                                value=seg_r.test_start_date.strftime(
                                    "%Y/%m/%d %H:%M:%S"
                                ),
                            )
                            sample.results.add(
                                code="BOTTLE_INTRODUCE_DATETIME",
                                value=seg_r.test_start_date.strftime("%Y%m%d%H%M%S"),
                            )
                            sample.results.add(
                                code="INTRODUCE_DATE",
                                value=seg_r.test_start_date.strftime("%Y/%m/%d"),
                            )
                            sample.results.add(
                                code="INTRODUCE_TIME",
                                value=seg_r.test_start_date.strftime("%H:%M:%S"),
                            )
                            sample.results.add(
                                code="INTRODUCTION_DATE",
                                value=seg_r.test_start_date.strftime("%d/%m/%Y"),
                            )
                            sample.results.add(
                                code="INTRODUCTION_HEURE",
                                value=seg_r.test_start_date.strftime("%H:%M"),
                            )

                        if seg_r.result_status_date:
                            # Heure et date du retrait du flacon
                            date_result_determinated = datetime.datetime.strptime(
                                seg_r.result_status_date, "%Y%m%d%H%M%S"
                            )
                            growth_delay = self.timedelta_to_hour_min(
                                date_result_determinated - seg_r.test_start_date,
                                sep=self.bactec_hour_min_sep,
                            )
                            sample.results.add(
                                code="DETERMINED_DATETIME",
                                value=date_result_determinated.strftime("%Y%m%d%H%M%S"),
                            )
                            sample.results.add(
                                code="DETERMINED_DATE",
                                value=date_result_determinated.strftime("%Y/%m/%d"),
                            )
                            sample.results.add(
                                code="DETERMINED_TIME",
                                value=date_result_determinated.strftime("%H:%M:%S"),
                            )
                            sample.results.add(
                                code="DATE_DETERMINATION",
                                value=date_result_determinated.strftime("%d/%m/%Y"),
                            )
                            sample.results.add(
                                code="HEURE_DETERMINATION",
                                value=date_result_determinated.strftime("%H:%M"),
                            )
                            if seg_r.test_status == "INST_POSITIVE":
                                # Date et heure de positivité
                                sample.results.add(
                                    code="POSITIVITY_DATETIME",
                                    value=date_result_determinated.strftime(
                                        "%Y%m%d%H%M%S"
                                    ),
                                )
                                # Date de positivité
                                sample.results.add(
                                    code="POSITIVITY_DATE",
                                    value=date_result_determinated.strftime("%Y/%m/%d"),
                                )
                                sample.results.add(
                                    code="POSITIVITE_DATE",
                                    value=date_result_determinated.strftime("%d/%m/%Y"),
                                )
                                # Heure de positivité
                                sample.results.add(
                                    code="POSITIVITY_TIME",
                                    value=date_result_determinated.strftime("%H:%M:%S"),
                                )
                                sample.results.add(
                                    code="POSITIVITE_HEURE",
                                    value=date_result_determinated.strftime("%H:%M"),
                                )
                                # Delai de positivité
                                if seg_r.result_status_date:
                                    delay_positivity = (
                                        date_result_determinated - seg_r.test_start_date
                                    )
                                    sample.results.add(
                                        code="TIME_TO_DETECTION_POSITIVITY",
                                        value=self.timedelta_to_hour_min(
                                            delay_positivity,
                                            sep=self.bactec_hour_min_sep,
                                        ),
                                    )
                        sample.results.add(
                            code="GROWTH_UNITS", value=seg_r.growth_units
                        )
                        sample.results.add(code="GROWTH_TIME", value=growth_delay)
                    elif seg_r.result_type_code.startswith("AST"):
                        sample.results.add(
                            code="{}_RACK".format(seg_r.antibiotic),
                            value=seg_r.instrument_location,
                        )
                        if seg_r.result_status_date:
                            # Heure et date du retrait du flacon
                            date_result_determinated = datetime.datetime.strptime(
                                seg_r.result_status_date, "%Y%m%d%H%M%S"
                            )
                            growth_delay = self.timedelta_to_hour_min(
                                date_result_determinated - seg_r.test_start_date,
                                sep=self.bactec_hour_min_sep,
                            )
                        sample.results.add(
                            code="{}_GROWTH_UNITS".format(seg_r.antibiotic),
                            value=seg_r.growth_units if seg_r.growth_units else None,
                        )
                        sample.results.add(
                            code="{}_GROWTH_TIME".format(seg_r.antibiotic),
                            value=growth_delay,
                        )
                        if seg_r.test_status == "INST_COMPLETE":
                            sample.results.add(
                                code="{}_CONCENTRATION".format(seg_r.antibiotic),
                                value=seg_r.antibiotic_conc,
                            )
                            sample.results.add(
                                code="{}_SENSITIVITY".format(seg_r.antibiotic),
                                value=seg_r.ast_susceptibility,
                            )
                            units = (
                                seg_r.antibiotic_conc_unit
                                if seg_r.antibiotic_conc_unit
                                else "ug/ml"
                            )
                            sample.results.add(
                                code="{}_UNITS".format(seg_r.antibiotic), value=units
                            )
                            atb = {
                                "antibiotic": "A_{}".format(seg_r.antibiotic),
                                "concentration": seg_r.antibiotic_conc,
                                "sensitivity": seg_r.ast_susceptibility,
                                "units": units,
                            }
                            liste_atb.append(atb)

                if liste_atb:
                    value = [
                        {"sub_code": seg_o.isolate_number, "antibiogram": liste_atb}
                    ]
                    sample.results.add(
                        code=seg_r.result_type_code,
                        value=value,
                        date=date,
                        instrument="{}-{}".format(
                            seg_r.instrument_type, seg_r.instrument_number
                        ),
                    )
        return worklist

    @staticmethod
    def timedelta_to_hour_min(timedelta, sep=""):
        if isinstance(timedelta, datetime.timedelta):
            seconds = timedelta.total_seconds()
        else:
            seconds = int(timedelta)
        hours = int(seconds // 3600)
        minutes = int(seconds // 60 % 60)
        return "{0:02d}{1}{2:02d}".format(hours, sep, minutes)
