# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class IDNowSegmentH(HeaderSegment):
    sender_type = HprimString((5, 1))
    software_version = HprimString((5, 2))
    device_id = HprimString((5, 3))
    device_name = HprimString((5, 4))
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class IDNowSegmentP(Segment):
    sample_id = HprimString(4)

    orders = HprimForeign("O", "patient")


class IDNowSegmentO(Segment):
    instrument_specimen_id = HprimString(4)
    test_type = HprimString((5, 4))
    specimen_type = HprimString((16, 1))
    test_cartridge_lot_number = HprimString((16, 3))
    qc_result = HprimString((16, 5))
    report_type = HprimString(26, default="F")

    results = HprimForeign("R", "order")


class IDNowSegmentR(Segment):
    result_value_name = HprimString((3, 4))
    value_source = HprimString((3, 5), default="M")
    value = HprimString(4)
    abnormal_flags = HprimString(7)
    operator = HprimString(11)
    date_start = HprimDate(12)
    date_completed = HprimDate(13)
    instrument_id = HprimString(14)


class IDNowSegmentL(Segment):
    termination_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "id_now", "label": "ID NOW ALERE ASTM"}
    SEGMENT_CLASS_MAP = {
        "H": IDNowSegmentH,
        "P": IDNowSegmentP,
        "O": IDNowSegmentO,
        "R": IDNowSegmentR,
        "L": IDNowSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            if seg_p.sample_id:
                sample_id = seg_p.sample_id
            for seg_o in seg_p.orders:
                if seg_o.specimen_type == "QC":
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=seg_o.qc_result,
                        control_lot=seg_o.test_cartridge_lot_number,
                        specimen=seg_o.specimen_type,
                    )
                else:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=sample_id,
                        specimen=seg_o.specimen_type,
                    )
                for seg_r in seg_o.results:
                    if seg_r.date_completed:
                        date = seg_r.date_completed
                    if seg_r.operator:
                        operator = seg_r.operator
                    flags = set(seg_r.abnormal_flags[:])
                    sample.results.add(
                        code=seg_r.result_value_name,
                        date=date,
                        value=seg_r.value,
                        flags=sorted(flags),
                        operator=operator,
                        instrument=seg_r.instrument_id,
                    )
        return worklist
