# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

from driverng import Worklist

parser_class = "m2000"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||ASI^1.0^s/n^H1P1O1R1Q1L1C1|||||My^Host||P|1|19930630000000
    Q|1|^SID1000||^^^ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "SID1000", "type": "patient"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&|||ASI^1.0^s/n^H1P1O1R1Q1L1C1|||||My^Host||P|1|19930630000000
    Q|1|^SID1000||^^^ALL||||||||O
    L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||MCA|||||||P
    Q|1|^SID1000||||||||||X
    L|1|N"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||MCA|||||||P
        P|1||036666666||NYME^ANO|NJF||F|||||PRESCRIBER||||||||||||^LOC
        O|1|123456||^^^A\\^^^B|A|20130603110342|20130513114242||||||||||||||||||O
        L|1|N"""
    assert result == expected


def test_encode_order_without_prescriber(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["physicians"] = []
    result = self.order(worklist=worklist)
    expected = """H|\\^&|||MCA|||||||P
        P|1||036666666||NYME^ANO|NJF||F|||||||||||||||||^LOC
        O|1|123456||^^^A\\^^^B|A|20130603110342|20130513114242||||||||||||||||||O
        L|1|N"""
    assert result == expected


def test_result_error_internal(self):
    result = self.decode(
        """
    H|\\^&|||m2000^8.1.9.0^275020287^H1P1O1R1C1L1|||||||P|1|20200415161830
    P|1
    O|1|622011054240|622011054240^AJO2FA6P^E2|^^^SARS-COV-2^SARS-COV-2|||||||||||||||||||||X
    C|1|I|4458 : Échec du contrôle interne.|I
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "622011054240",
                "type": "patient",
                "rack": "AJO2FA6P-E2",
                "results": [
                    {"code": "SARS-COV-2", "flags": ["4458"], "value": "ERREUR"}
                ],
            }
        ],
    }
    assert result == expected


def test_result(self):
    date = datetime.datetime(2013, 7, 9, 14, 37, 53)
    result = self.decode(
        """
    H|\\^&|||m2000^5.0.188.0^275020646^H1P1O1R1C1L1|||||||P|1|20130703111108
    P|1
    O|1|10703998901|10703998901^hpv180613^B3|^^^0.4mlHRHPV^HR HPV|||||||||||||||||||||F
    R|1|^^^0.4mlHRHPV^HR HPV^96555HN01^10274201^^F|HPV 18 (31,52); Other HR HPV (28,58); HPV 16 (19,31)|||||F||labo^labo||20130709143753|275020646
    R|2|^^^0.5mlHRHPV^HR HPV^96555HN01^10274201^^F|Other HR HPV (33,44)|||||F||labo^labo||20130709143753|275020646
    R|3|^^^0.4mlHRHPV^HR HPV^96555HN01^10274201^^I|HR HPV Detected|||||F||labo^labo||20130709143753|275020646
    R|4|^^^NG^NG^444013^10274201^^F|Negative|DC||||R||labo^labo||20130709143753|275020646
    R|5|^^^NG^NG^444013^10274201^^I|Negative|||||R||labo^labo||20130709143753|275020646
    R|6|^^^NG^NG^444013^10274201^^P|-1.00|cycle number||||R||labo^labo||20130709143753|275020646
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "10703998901",
                "type": "patient",
                "rack": "hpv180613-B3",
                "results": [
                    {
                        "date": date,
                        "value": "31,52",
                        "code": "0.4mlHRHPVF_HPV_18",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "28,58",
                        "code": "0.4mlHRHPVF_Other_HR_HPV",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "19,31",
                        "code": "0.4mlHRHPVF_HPV_16",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "33,44",
                        "code": "0.5mlHRHPVF_Other_HR_HPV",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "HR HPV Detected",
                        "code": "0.4mlHRHPVI",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "Negative",
                        "code": "NGF",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "Negative",
                        "code": "NGI",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "date": date,
                        "value": "-1.00",
                        "code": "NGP",
                        "operator": "labo",
                        "comments": [],
                        "flags": [],
                    },
                ],
            }
        ],
    }
    assert result == expected
